/* SPDX-FileCopyrightText: 2016-2020 - Sébastien Wilmet <swilmet@gnome.org>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#ifndef TEPL_H
#define TEPL_H

#define TEPL_H_INSIDE

#include <amtk/amtk.h>

#include <tepl/tepl-enum-types.h>
#include <tepl/tepl-macros.h>

#include <tepl/tepl-abstract-factory.h>
#include <tepl/tepl-application.h>
#include <tepl/tepl-application-window.h>
#include <tepl/tepl-buffer.h>
#include <tepl/tepl-file.h>
#include <tepl/tepl-file-chooser.h>
#include <tepl/tepl-file-loader.h>
#include <tepl/tepl-file-saver.h>
#include <tepl/tepl-fold-region.h>
#include <tepl/tepl-goto-line-bar.h>
#include <tepl/tepl-gutter-renderer-folds.h>
#include <tepl/tepl-info-bar.h>
#include <tepl/tepl-init.h>
#include <tepl/tepl-io-error-info-bars.h>
#include <tepl/tepl-iter.h>
#include <tepl/tepl-menu-shell.h>
#include <tepl/tepl-metadata.h>
#include <tepl/tepl-metadata-manager.h>
#include <tepl/tepl-notebook.h>
#include <tepl/tepl-panel.h>
#include <tepl/tepl-statusbar.h>
#include <tepl/tepl-style-scheme-chooser-widget.h>
#include <tepl/tepl-tab.h>
#include <tepl/tepl-tab-group.h>
#include <tepl/tepl-tab-label.h>
#include <tepl/tepl-tab-loading.h>
#include <tepl/tepl-tab-saving.h>
#include <tepl/tepl-utils.h>
#include <tepl/tepl-view.h>

#undef TEPL_H_INSIDE

#endif /* TEPL_H */
